@@ -36,7 +36,6 @@ import sys
 import tempfile
 
 import networkx as nx
-from networkx.utils import _get_fh,is_string_like
 
 
 def from_agraph(A,create_using=None):
@@ -86,24 +85,22 @@ def from_agraph(A,create_using=None):
     # assign defaults
     N=nx.empty_graph(0,create_using)
     N.name=str(A)
-    node_attr={}
     # add nodes, attributes to N.node_attr
     for n in A.nodes():
-        N.add_node(str(n),**dict(n.attr))
+        str_attr=dict((str(k),v) for k,v in n.attr.items())
+        N.add_node(str(n),**str_attr)
 
     # add edges, assign edge data as dictionary of attributes
     for e in A.edges():
         u,v=str(e[0]),str(e[1])
         attr=dict(e.attr)
-        if N.is_multigraph():
-            if e.key is not None:
-                attr[key]=e.key
-            N.add_edge(u,v,**attr)
+        str_attr=dict((str(k),v) for k,v in attr.items())
+        if not N.is_multigraph():
+            if e.name is not None:
+                str_attr['key']=e.name
+            N.add_edge(u,v,**str_attr)
         else:
-            if e.key is not None:
-                N.add_edge(u,v,e.key,**attr)
-            else:
-                N.add_edge(u,v,**attr)
+            N.add_edge(u,v,key=e.name,**str_attr)
 
     # add default attributes for graph, nodes, and edges
     # hang them on N.graph_attr
@@ -135,7 +132,7 @@ def to_agraph(N):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError("to_agraph() requires pygraphviz: http://nx.lanl.gov/pygraphviz")
+        raise ImportError("to_agraph() requires pygraphviz: http://nx.lanl.gov/pygraphviz (not available for Python3")
     directed=N.is_directed()
     strict=N.number_of_selfloops()==0 and not N.is_multigraph()
     A=pygraphviz.AGraph(name=N.name,strict=strict,directed=directed)
@@ -177,7 +174,7 @@ def write_dot(G,path):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError("write_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz")
+        raise ImportError("write_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3")
 
     A=to_agraph(G)
     A.write(path)
@@ -195,7 +192,7 @@ def read_dot(path):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError("read_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz")
+        raise ImportError("read_dot() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3)")
     A=pygraphviz.AGraph(file=path)
     return from_agraph(A)
 
@@ -257,7 +254,7 @@ def pygraphviz_layout(G,prog='neato',root=None, args=''):
     try:
         import pygraphviz
     except ImportError:
-        raise ImportError("pygraphviz_layout() requires pygraphviz: http://networkx.lanl.gov/pygraphviz")
+        raise ImportError("pygraphviz_layout() requires pygraphviz: http://networkx.lanl.gov/pygraphviz (not available for Python3")
     A=to_agraph(G)
     if root is not None:
         args+="-Groot=%s"%root
@@ -354,9 +351,9 @@ def view_pygraphviz(G, edgelabel=None, prog='neato', args='',
             del G.graph[which]
 
     # Update all default values
-    added_edge = update_attrs('edge', edge_attrs)
-    added_node = update_attrs('node', node_attrs)
-    added_graph = update_attrs('graph', graph_attrs)
+    update_attrs('edge', edge_attrs)
+    update_attrs('node', node_attrs)
+    update_attrs('graph', graph_attrs)
 
     # Convert to agraph, so we inherit default values
     A = to_agraph(G)
